/*
   PkgsSel-data
   Copyright (C) 1999 Jarek Woloszyn (yossa@pld.org.pl)
   $Id: pkgssel-data.c,v 1.3 2001/02/11 17:35:57 yossa Exp $
    
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
 */


#include <string.h>
#include <newt.h>
#include <trurl/trurl.h>
#include <rpmpldi.h>

#include "utils.h"
#include "pkgssel-data.h"

tn_array *selGroups = NULL;

/* This file is readed using fopen, fgets, fclose. It should be changed
 * to use iointf from pldilib (i think)
 */
#define GROUP_FILE "./groups"

/* Function for sorting and bsearching - entries are sorted by type 
 * (directories first) and then by names
 */

static int cmp_names_and_types(const void *a, const void *b)
{
    TSelectorItem *i = (TSelectorItem *) a;
    TSelectorItem *j = (TSelectorItem *) b;
    int flag = SEL_DIR | SEL_PACKET;
    if ((i->type & flag) == (j->type & flag))
	return strcmp(i->nameptr, j->nameptr);
    else if ((i->type & SEL_PACKET) == SEL_PACKET)
	return 1;
    else
	return -1;
}

/* Free the selector structure (we have to free all dir entries */
static void free_list(void *p)
{
    TSelectorItem *i = (TSelectorItem *) p;
    if ((i->type & SEL_DIR) == SEL_DIR) {
	n_array_free(i->dir);
	free(i->nameptr);
    }
    free(p);
}


/* Adds packet to the structure.  
 */

TSelectorItem *
 AddPacket(tn_array * l, TPkgSet * pkgsSet, int num)
{
    TSelectorItem *item;
    TPkgInfo *pkg;
    
    pkg = pkgs_array_nth(pkgsSet, num);
    item = (TSelectorItem *) malloc(sizeof(TSelectorItem));
    item->type = SEL_PACKET;
    item->nameptr = pkg->name;
    item->selptr = &pkg->selected;
    item->pkgnum = num;
    item->ver = (char *) rpmpldi_header_entry_get(num, RPMTAG_VERSION);
    item->release = (char *) rpmpldi_header_entry_get(num, RPMTAG_RELEASE);
    item->epoch = (int *) rpmpldi_header_entry_get(num, RPMTAG_EPOCH);
    item->dir = NULL;
    if (n_array_push(l, (void *) item) == NULL) {
	log_message(_("%s: Error in n_array_push"), __FUNCTION__);
	return NULL;
    }
    
    /* Keep entries sorted all the time */
    n_array_sort_ex(l, cmp_names_and_types);
    return item;
}

/* Adds directory entry and creates a subdirectory */
static tn_array *AddDir(tn_array * l, char *name, int active)
{
    TSelectorItem *item;
    item = (TSelectorItem *) malloc(sizeof(TSelectorItem));

    item->type = SEL_DIR | (active ? SEL_ACTIVE : 0);
    item->nameptr = strdup(name);
    item->selptr = NULL;
    item->pkgnum = 0;
    item->ver = NULL;
    item->release = NULL;
    item->epoch = NULL;
    /* create subdir */
    item->dir = n_array_new(10, free_list, cmp_names_and_types);
    if (n_array_push(l, (void *) item) == NULL) {
	log_message(_("%s: Error in n_array_push"), __FUNCTION__);
	return NULL;
    }
    n_array_sort_ex(l, cmp_names_and_types);
    return item->dir;
}

/*  Looks for name in current group  */
static TSelectorItem *FindDir(tn_array * l, char *name)
{
    TSelectorItem item;
    TSelectorItem *i = &item;
    i->nameptr = name;
    i->type = SEL_DIR;
    n_array_sort_ex(l, cmp_names_and_types);
    i = (TSelectorItem *) n_array_bsearch_ex
	(l, (void *) i, cmp_names_and_types);
    return i;
}

/*  Creates a hierarchy of dirs */
static tn_array *CreateDir(tn_array * item, char *pkggroup)
{
    TSelectorItem *foundPtr;
    tn_array *tmpPtr;
    char *groupname;
    char tmpstr[256];
    n_assert(pkggroup != NULL);
    n_assert(strcmp(pkggroup, ""));

    strcpy(tmpstr, pkggroup);
    groupname = strtok(tmpstr, "/");
    tmpPtr = item;
    /* Create hierarchy */
    while (groupname != NULL) {
	/* Create the group if it doesn't exist */
	if ((foundPtr = FindDir(tmpPtr, groupname)) == NULL) {
	    tmpPtr = AddDir(tmpPtr, groupname, 0);
	} else
	    tmpPtr = foundPtr->dir;
	groupname = strtok(NULL, "/");
    }
    return tmpPtr;
}

/* Fills the main list with list of currently selected packages
 */
void pkgsCreateActive(tn_array * l)
{
    int i;
    TPkgSet *pkgsSet;
    pkgsSet = rpmpldi_getPkgsSet();

    for (i = 0; i < pkgs_array_num(pkgsSet); i++) {
	if (pkgs_array_nth(pkgsSet, i)->selected)
	    AddPacket(l, pkgsSet, i);
    }
}


/* Loads the selector groups. They should exactly in format presented
 * by groups file included in the package
 */
static int loadFileGroups(tn_array * groups, TPkgSet * pkgsSet)
{
    FILE *fstr;
    char line[256], *line_tmp;	/* current line i file */
    char *ptr;
    int select=0;			/* select this group? */
    int i, j;
    tn_array *groupPtr = NULL;

    if ((fstr = fopen(GROUP_FILE, "r")) == NULL) {
	log_message(_("%s: Cannot open %s file."), __FUNCTION__, GROUP_FILE);
	return 0;
    }
    while (fgets(line, 255, fstr) != NULL) {
	/* group names are preceded by asterix (*) */
	if (line[0] == '*') {
	    ptr = strtok(line, " ");
	    ptr = strtok(NULL, " ");
	    if (*ptr == '1')
		select = 1;
	    else
		select = 0;
	    ptr = strtok(NULL, "\n");
	    groupPtr = AddDir(groups, ptr, 0);
	} else {
	    n_assert(groupPtr != NULL);
	    for (line_tmp = line; *line_tmp == ' '; line_tmp++);
	    if (line_tmp[strlen(line_tmp) - 1] == '\n')
		line_tmp[strlen(line_tmp) - 1] = 0;
	    for (; line_tmp[strlen(line_tmp) - 1] == ' ';
		 line_tmp[strlen(line_tmp) - 1] = 0);
	    /* check if the package is in tocfile 
	     * add all versions of package 
	     */
	    for (i = 0, j = 0; i < pkgs_array_num(pkgsSet); i++) {
		if (strcmp(pkgs_array_nth(pkgsSet, i)->name, line_tmp) == 0 &&
		    strlen(line_tmp) > 1) {
		    AddPacket(groupPtr, pkgsSet, i);
		    pkgs_array_nth(pkgsSet, i)->selected = select;
		    if (j)
			pkgs_array_nth(pkgsSet, j)->selected = 0;
		    j = i;
		}
	    }
	}
    }
    fclose(fstr);
    return 1;
}

/* Loads lists of packets */
int pkgsLoadGroups()
{
    TPkgSet *pkgsSet;
    tn_array *namePtr, *groupPtr, *tmpPtr,*upgradeAll,*upgradeVer;
    tn_array *notInstalled;
    char *pkggroup;

    int i;
    pkgsSet = rpmpldi_getPkgsSet();

    /*  Create root directory */
    selGroups = n_array_new(10, free_list, cmp_names_and_types);

    namePtr = AddDir(selGroups, _("by-name"), 0);
    groupPtr = AddDir(selGroups, _("by-rpm-group"), 0);
    upgradeAll = AddDir(selGroups,_("upgradable"),0);
    upgradeVer = AddDir(selGroups,_("upgradable-new-versions"),0);
    notInstalled = AddDir(selGroups, _("not-installed"),0);
    
    for (i = 0; i < pkgs_array_num(pkgsSet); i++) {
	log_debug_message(_("Processing %s"),pkgs_array_nth(pkgsSet,i)->name);
	/*  Create alphabetic list */
	AddPacket(namePtr, pkgsSet, i);

	/* Create list of upgreatable pacakges */
	if(pkgs_array_nth(pkgsSet,i)->flags&TPKGINF_UPGRADE){
		AddPacket(upgradeAll,pkgsSet,i);
		if(!(pkgs_array_nth(pkgsSet,i)->flags&
					TPKGINF_DIFF_RELEASE))
			AddPacket(upgradeVer,pkgsSet,i);
	}

	/* list of not installed packages */
	if(!(pkgs_array_nth(pkgsSet,i)->flags&(TPKGINF_EQUAL|
					TPKGINF_UPGRADE|
					TPKGINF_OLDER)))
		AddPacket(notInstalled,pkgsSet,i);
	

	/*  Create rpm group list */
	pkggroup = (char *) rpmpldi_header_entry_get(i, RPMTAG_GROUP);
	if (!strcmp(pkggroup,""))
		  pkggroup=_("Unknown");
	tmpPtr = CreateDir(groupPtr, pkggroup);
	AddPacket(tmpPtr, pkgsSet, i);
    }

    groupPtr = AddDir(selGroups, _("selected-packages"), 1);
    groupPtr = AddDir(selGroups, _("by-selector-group"), 0);
    if (!loadFileGroups(groupPtr, pkgsSet)) {
/*	n_array_free(selGroups);
	return 0;
works even without group file        
*/
    }
    return 1;
}

void pkgsFreeGroups()
{
    n_array_free(selGroups);
}
